"""Representation of a localized bundle."""

from typing import cast, Dict, List, Set

from dotstrings.localized_string import LocalizedString


class LocalizedBundle:
    """Representation of a localized bundle.

    :param raw_entries: The raw dictionary entries that were parsed from disk
    """

    def __init__(self, raw_entries: Dict[str, Dict[str, List[LocalizedString]]]) -> None:
        self.raw_entries = raw_entries

    def languages(self) -> List[str]:
        """Return the languages supported in the bundle

        :returns: A list of language codes
        """
        return list(self.raw_entries.keys())

    def table_names(self, validate_identical: bool = False) -> List[str]:
        """Return the tables in the bundle.

        :param validate_identical: Set to True to confirm all languages have the
                                   same tables in them

        :returns: A list of table names
        """

        if validate_identical:

            # Build up a map of languages to table names

            found_tables: Dict[str, Set[str]] = {}
            for language, table_map in self.raw_entries.items():
                # table_map is a dictionary of names to lists of strings
                found_tables[language] = set(table_map.keys())

            if len(found_tables) == 0:
                return []

            base_language = list(found_tables.keys())[0]
            base_language_tables = found_tables[base_language]

            for language, table_names in found_tables.items():
                extra_tables = table_names - base_language_tables
                missing_tables = base_language_tables - table_names

                if len(extra_tables) > 0:
                    raise Exception(
                        f"The following table names were in {language}"
                        + f" but not in {base_language}: {extra_tables}"
                    )

                if len(missing_tables) > 0:
                    raise Exception(
                        f"The following table names were in {base_language}"
                        + f" but not in {language}: {missing_tables}"
                    )

            return list(base_language_tables)

        all_table_names = set()

        for table_map in self.raw_entries.values():
            for table in table_map.keys():
                all_table_names.add(table)

        return list(all_table_names)

    def tables_for_language(self, language: str) -> Dict[str, List[LocalizedString]]:
        """Return the tables for a language.

        :param language: The language to get the tables for

        :returns: A dictionary of table names to lists of strings
        """
        sentinel = object()
        result = self.raw_entries.get(language, sentinel)

        if result is sentinel:
            raise Exception(f"There were no entries for language: {language}")

        return cast(Dict[str, List[LocalizedString]], result)

    def table_for_languages(
        self, table: str, *, allow_missing: bool = False
    ) -> Dict[str, List[LocalizedString]]:
        """Return a dictionary of languages to strings for a given table.

        :param table: The table to load the data for
        :param bool allow_missing: Set to True to allow a table to be missing for any languages

        :returns: A dictionary of language codes to a list of strings
        """
        results = {}

        for language, table_map in self.raw_entries.items():
            sentinel = object()
            table_data = table_map.get(table, sentinel)

            if table_data is sentinel and language == "Base":
                continue
            elif table_data is sentinel and not allow_missing:
                raise Exception(f"Could not find table {table} for language {language}")

            results[language] = cast(List[LocalizedString], table_data)

        return results

    def tables(
        self, *, validate_missing: bool = True
    ) -> Dict[str, Dict[str, List[LocalizedString]]]:
        """Return the entries in the bundle, first keyed by table, then by language.

        :param bool validate_missing: Set to False to disable the check that a table exists for every language

        Example response:

        ```
        {
            "MyTable": {
                "en": [x, y, z],
                "fr": [x, y, z]
            },
            "OtherTable": {
                "en": [a, b, c],
                "fr": [a, b, c]
            }
        }
        ```

        :returns: A dictionary of table names to a dictionary of languages to strings
        """
        results = {}

        for table_name in self.table_names():
            results[table_name] = self.table_for_languages(
                table_name, allow_missing=(not validate_missing)
            )

        return results

    def merge_bundle(self, other: "LocalizedBundle") -> None:
        """Merge another bundle into this one.

        Any identical tables results in an exception

        :param other: The other bundle to merge into this one

        :raises Exception: If the other bundle has identical tables
        """

        for language, table_map in other.raw_entries.items():
            for table, strings in table_map.items():
                if self.raw_entries.get(language) is None:
                    self.raw_entries[language] = {}
                self.raw_entries[language][table] = strings



#!/usr/bin/env python3

"""Utilities for dealing with .strings files"""

import re
from typing import List, Match, Optional, TextIO, Tuple, Union

from dotstrings.dot_strings_entry import DotStringsEntry


_ENTRY_REGEX = r'^"(.+)"\s?=\s?"(.*)";$'
_ENTRY_PATTERN = re.compile(_ENTRY_REGEX)

_NS_ENTRY_REGEX = r'^(NS[^ ]+)\s?=\s?"(.*)";$'
_NS_ENTRY_PATTERN = re.compile(_NS_ENTRY_REGEX)


def load(file_details: Union[TextIO, str], encoding: Optional[str] = None) -> List[DotStringsEntry]:
    """Parse the contents of a .strings file from a file pointer.

    :param file_details: The file pointer or a file path
    :param encoding: The encoding the file is in

    :returns: A list of `DotStringEntry`s
    """

    # If it's a file pointer, read in the contents and parse
    if not isinstance(file_details, str):
        contents = "".join(file_details.readlines())
        return loads(contents)

    # It must have been a path instead then, so open the file, and parse
    if encoding:
        encoding_list = [encoding]
    else:
        encoding_list = ["utf-8", "utf-16-le", "utf-16-be"]

    for encoding_option in encoding_list:
        try:
            with open(file_details, "r", encoding=encoding_option) as strings_file:
                return load(strings_file)
        except UnicodeDecodeError:
            pass

    raise Exception(f"Could not determine encoding for file at path: {file_details}")


def loads(contents: str) -> List[DotStringsEntry]:
    """Parse the contents of a .strings file.

    Note: CRLF is not supported in strings.

    :param contents: The contents of a .strings file

    :returns: A list of `DotStringsEntry`s"""

    # Sometimes we have CRLF. It's easier to just replace now. This could, in
    # theory, cause issues, but we just don't support it for now.
    if "\r\n" in contents:
        raise Exception("Strings contain CRLF")
    contents = contents.replace("\r\n", "\n")

    # Let's split so that we have a single item and it's comments together
    entries = contents.split('";')

    # Remove any empty entries
    entries = [entry for entry in entries if len(entry.strip()) > 0]

    # Add the splitter back on the end
    entries = [entry + '";' for entry in entries]

    parsed_entries: List[DotStringsEntry] = []

    # Now we can work on parsing them one by one
    for entry in entries:
        parsed_entries.append(_parse_entry(entry))

    return parsed_entries


def _find_entry(entry_lines: List[str]) -> Tuple[int, Optional[Match]]:
    """Search for the entry in some entry lines

    :param entry_lines: The lines for an entry from a .strings file (including comments)

    :returns: A tuple with the index of the match and the match itself
    """

    for index, line in enumerate(entry_lines):

        # Naive checks to avoid doing a regex if we don't have to
        if len(line) == 0:
            continue

        if len(line.strip()) == 0:
            continue

        if line.startswith(" "):
            continue

        match = _ENTRY_PATTERN.match(line)
        if match:
            return index, match

        # We didn't match so try the NS entry one
        match = _NS_ENTRY_PATTERN.match(line)
        if match:
            return index, match

    return 0, None


def _parse_entry(entry: str) -> DotStringsEntry:
    """Parse a single entry in a .strings file and its comments.

    :param entry: A single entry from a .strings file

    :returns: A parsed entry value

    :raises Exception: If we fail to parse the entry
    """

    # pylint: disable=too-many-branches

    lines = entry.split("\n")

    entry_index, entry_match = _find_entry(lines)

    # If we didn't find it, then that's a problem
    if entry_match is None:
        raise Exception("Failed to find key and value in entry:\n" + entry)

    # We also expect it to be the last line, so if it's not, then that's a problem too
    if entry_index != len(lines) - 1:
        raise Exception("Found key and value in an unexpected position in entry:\n" + entry)

    # We now have the key and value
    key = entry_match.group(1)
    value = entry_match.group(2)

    # Just the comment to go

    # We already know the key and value were on the last line, so let's drop it
    lines = lines[:-1]

    comment = ""
    in_comment = False

    for line in lines:

        if not in_comment and "/*" in line:
            in_comment = True

        if not in_comment:
            continue

        if line.strip().startswith("/*"):
            line = line.replace("/*", "")

        if line.strip().endswith("*/"):
            line = line.replace("*/", "")

        comment += line.strip() + "\n"

        if "*/" in line:
            in_comment = False

    # If we didn't find any comment, set it to None
    if len(comment) == 0:
        return DotStringsEntry(key, value, [])

    comments = comment.split("\n")
    comments = [comment.strip() for comment in comments]
    comments = [comment for comment in comments if len(comment) > 0]

    return DotStringsEntry(key, value, comments)

    # pylint: enable=too-many-branches



"""Utilities for dealing with .strings files"""

import os
from typing import Dict, List, Optional, Set

from dotstrings.parser import load, loads
from dotstrings.dot_strings_entry import DotStringsEntry
from dotstrings.localized_bundle import LocalizedBundle
from dotstrings.localized_string import LocalizedString


def language_folder_path(strings_folder: str, language: str) -> str:
    """Generate the folder path for a language.

    :param strings_folder: The location of the strings folder (which contains
                           all the *.lproj folders)
    :param language: The language code for the file

    :returns: The path to the strings file
    """
    return os.path.join(strings_folder, f"{language}.lproj")


def strings_file_path(strings_folder: str, language: str, table_name: str) -> str:
    """Generate the .strings file path from the details supplied.

    :param strings_folder: The location of the strings folder (which contains
                           all the *.lproj folders)
    :param language: The language code for the file
    :param table_name: The name of the table (file name)

    :returns: The path to the strings file
    """
    return os.path.join(language_folder_path(strings_folder, language), f"{table_name}.strings")


def languages_in_folder(strings_folder: str) -> Set[str]:
    """Find all the languages in a folder

    This looks for *.lproj folders.

    :param strings_folder: The location of the strings folder (which contains
                           all the *.lproj folders)

    :returns: A set containing all the languages.
    """
    languages = os.listdir(strings_folder)
    languages = [language for language in languages if language.endswith(".lproj")]
    return {language.replace(".lproj", "") for language in languages}


def load_table(strings_folder: str, language: str, table_name: str) -> List[LocalizedString]:
    """Load the specified .strings table

    :param strings_folder: The location of the strings folder (which contains
                           all the *.lproj folders)
    :param language: The language code to load
    :param table_name: The name of the table to load

    :returns: A list of strings in the table
    """

    table_path = strings_file_path(strings_folder, language, table_name)
    strings = load(table_path)
    return LocalizedString.from_dotstring_entries(
        entries=strings, language=language, table=table_name
    )


def load_language_tables(strings_folder: str, language: str) -> Dict[str, List[LocalizedString]]:
    """Load the .strings tables for a given language

    :param strings_folder: The location of the strings folder (which contains
                           all the *.lproj folders)
    :param language: The language code to load

    :returns: A dictionary of results. The key is the table, the value is the
              list of strings in the table
    """

    language_folder = language_folder_path(strings_folder, language)

    results = {}

    for table in os.listdir(language_folder):
        if not table.endswith(".strings"):
            continue
        table_name = table.replace(".strings", "")
        results[table_name] = load_table(strings_folder, language, table_name)

    return results


def load_all_strings(strings_folder: str) -> LocalizedBundle:
    """Load the .strings tables for a given language

    :param strings_folder: The location of the strings folder (which contains
                           all the *.lproj folders)
    :param language: The language code to load

    :returns: A LocalizedBundle containing the results
    """

    languages = languages_in_folder(strings_folder)

    results = {}

    for language in sorted(languages):
        results[language] = load_language_tables(strings_folder, language)

    return LocalizedBundle(results)


def normalize(
    strings_path: str,
    output_path: Optional[str] = None,
    remove_duplicates: bool = True,
    sort_comments: bool = True,
) -> None:
    """Load in the specified .strings table, normalize it, and write it back out.

    Normalizing consists of sorting by key, then comments, and (usually)
    deduplicating before writing back out.

    :param strings_path: The location of the strings file
    :param output_path: The location to write the sorted file to. If None, it
                        will overwrite the input file.
    :param remove_duplicates: By default, any duplicates will be removed and
                              their comments combined. Set to False to raise
                              an exception instead
    :param sort_comments: By default comments will be sorted in alphabetical
                          order. However, if you use linebreaks in your comments
                          you will wish to turn this off. It also removes
                          duplicates.
    """

    entries = load(strings_path)
    entries.sort(key=lambda x: [x.key] + x.comments)

    deduped_entries = [entries[0]]

    for entry in entries[1:]:

        # If the keys are different, just add to the list and move on
        if deduped_entries[-1].key != entry.key:
            deduped_entries.append(entry)
            continue

        # If we have duplicate keys but the values don't match, that's an
        # exception, whether or not we are removing duplicates
        if deduped_entries[-1].value != entry.value or not remove_duplicates:
            raise Exception(f"Found duplicate strings with key: {entry.key}")

        deduped_entries[-1].comments.extend(entry.comments)

    if sort_comments:
        for entry in deduped_entries:
            entry.comments = list(set(entry.comments))
            entry.comments.sort()

    if output_path is None:
        output_path = strings_path

    with open(output_path, "w") as strings_file:
        for entry in deduped_entries:
            strings_file.write(entry.strings_format())
            strings_file.write("\n\n")



"""Base types for the dotstrings library."""

from typing import List


class DotStringsEntry:
    """Represents a .strings entry.

    :param key: The key for the entry
    :param value: The value for the entry
    :param comments: Any comments associated with the entry
    """

    key: str
    value: str
    comments: List[str]

    def __init__(self, key: str, value: str, comments: List[str]) -> None:
        self.key = key
        self.value = value
        self.comments = comments

    def strings_format(self) -> str:
        """Return the entry as would be formatted in a .strings file

        :returns: The .strings representation of this entry
        """
        key_value = f'"{self.key}" = "{self.value}";'

        if len(self.comments) == 0:
            return key_value

        if len(self.comments) == 1:
            return f"/* {self.comments[0]} */\n{key_value}"

        if len(self.comments) == 2:
            return f"/* {self.comments[0]}\n   {self.comments[1]} */\n{key_value}"

        output = f"/* {self.comments[0]}\n"

        for comment in self.comments[1:-1]:
            output += f"   {comment}\n"

        output += f"   {self.comments[-1]} */\n{key_value}"

        return output

    def __repr__(self) -> str:
        """Returns a raw representation of the object which can be used to reconstruct it later.

        :returns: A raw representation of the object
        """
        return str({"key": self.key, "value": self.value, "comments": self.comments})

    def __str__(self) -> str:
        """Generate and return the string representation of the object.

        :return: A string representation of the object
        """
        return self.__repr__()



"""Wrapper around the genstrings command"""

import os
import shutil
import subprocess
import tempfile
from typing import List


def _convert_to_utf8(file_path: str) -> None:
    """Take a UTF-16 file and convert to UTF-8.

    NOTE: This will replace the existing file

    :param file_path: The path of the file to convert

    :raises Exception: If we are unable to convert
    """

    temp_file_path = tempfile.mktemp()

    iconv_command = f'iconv -f UTF-16 -t UTF-8 "{file_path}" > "{temp_file_path}"'

    if subprocess.run(iconv_command, shell=True).returncode != 0:
        raise Exception("Unable to convert from UTF-16 to UTF-8!")

    shutil.move(temp_file_path, file_path)


def generate_strings(
    *, output_directory: str, file_paths: List[str], clear_existing: bool = True
) -> None:
    """Run the genstrings command over the files passed in.

    Genstrings scans code files for usage of the `NSLocalizedString` macro. It
    the generates the corresponding .strings file from these. e.g. If you have:

    ```objc
    label.text = NSLocalizedString(@"Hello World", @"Greeting to the user");
    ```

    then the tool will find this and generate an `en.lproj/Localizable.strings`
    file with the content:

    ```
    /* Greeting to the user */
    "Hello World" = "Hello World";
    ```

    However, if you specify a table in your `NSLocalizedString` call, then
    instead of using the default `Localizable.strings` file, it will generate
    `MyTable.strings`.

    :param str output_directory: The location to place the output files (this
                                 folder will contain an en.lproj folder after)
    :param List[str] file_paths: The paths to the files that should be scanned
    :param bool clear_existing: Set to True when the existing files in the
                                output directory should be wiped before
                                generating the new strings. Defaults to True.

    :raises Exception: If we can't generate the .strings files
    """

    # Determine output paths
    english_strings_directory = os.path.join(output_directory, "en.lproj")

    # Create output directory
    os.makedirs(english_strings_directory, exist_ok=True)

    # Empty existing strings
    if clear_existing:
        for table in os.listdir(english_strings_directory):
            with open(os.path.join(english_strings_directory, table), "w") as table_file:
                table_file.write("")

    # We can't pass in too many files on the command line or the argument list
    # is too long. To avoid this, we do it in chunks of 100.
    files_per_iteration = 100

    for i in range(0, (len(file_paths) // files_per_iteration) + 1):

        # Take the next group of files
        current_files = file_paths[i * files_per_iteration : (i + 1) * files_per_iteration]

        genstrings_command = ["xcrun", "extractLocStrings", "-a", "-noPositionalParameters", "-u"]
        genstrings_command += ["-o", english_strings_directory]
        genstrings_command.extend(current_files)

        try:
            output = subprocess.run(
                genstrings_command,
                universal_newlines=True,
                check=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
            ).stdout

            output = output.strip()

            if len(output) > 0:
                raise Exception(f"Encountered an error generating strings: {output}")
        except subprocess.CalledProcessError as ex:
            raise Exception(f"Unable generate .strings files! {ex}")

    # Convert all .strings files to UTF-8
    for file_name in os.listdir(english_strings_directory):
        file_path = os.path.join(english_strings_directory, file_name)
        # Check for file type and .strings extension
        if file_name.endswith(".strings") and os.path.isfile(file_path):
            _convert_to_utf8(file_path)



"""Localized string types"""

import hashlib
import re
from typing import ClassVar, List, Optional, Pattern

from dotstrings.dot_strings_entry import DotStringsEntry


class LocalizedString:
    """Represents a localized string.

    Note: Automatic key generation is supported. By this, if you pass None as
    the value for your `key`, the value will automatically be derived as a hash
    of the value and the key extension. This should normally only ever be done
    for your default language (as the key will change between languages).

    The reason we support this is that this is a useful class to use in various
    projects, not just as the result of reading .strings files. In those cases,
    you may wish to automatically generate keys to avoid having to manually
    deal with deduplication.

    :param Optional[str] key: The key for the string. If this is None, they key
                              will be automatically derived from the value and
                              the key extension.
    :param str value: The value of the string
    :param str language: The language code of the string
    :param str table: The string table to use
    :param Optional[str] comment: The comment for the string
    :param Optional[str] key_extension: The key extension to differentiate
                                        between identical strings with different
                                        meanings
    :param str bundle: The bundle the string can be found in
    """

    _TOKEN_REGEX: ClassVar[
        str
    ] = r"(%(?:[0-9]+\$)?[0-9]*\.?[0-9]*[a-zA-Z]{0,2}[dDuUxXoOfFeEgGcCsSaAp@])"
    _TOKEN_PATTERN: ClassVar[Pattern] = re.compile(_TOKEN_REGEX, flags=re.DOTALL)

    key: str
    value: str
    language: str
    table: str
    comment: Optional[str]
    key_extension: Optional[str]
    bundle: str

    def __init__(
        self,
        *,
        key: Optional[str],
        value: str,
        language: str,
        table: str,
        comment: Optional[str] = None,
        key_extension: Optional[str] = None,
        bundle: str = "",
    ) -> None:
        self.value = value
        self.language = language
        self.comment = comment
        self.key_extension = key_extension
        self.table = table
        self.bundle = bundle

        if key is not None:
            self.key = key
        else:
            self.key = LocalizedString._calculate_key(value=value, key_extension=key_extension)

    @staticmethod
    def _calculate_key(*, value: str, key_extension: Optional[str]) -> str:
        """Calculate the unique key to use for this string.

        :param value: The value of the localized string
        :param key_extension: Any key extension value to use to differentiate
                              between identical values

        :returns: The unique key for the string
        """

        # Calculate key based on string interpreted by the compiler
        hash_input = value

        if key_extension is not None:
            hash_input += ":" + key_extension

        hash_input = hash_input.replace("\\n", "\n")  # Replace "slash n" with newline character
        hash_input = hash_input.replace('\\"', '"')  # Replace "slash quote" with quote character

        key = hashlib.md5(hash_input.encode("utf-8")).hexdigest()

        return key

    def tokens(self) -> List[str]:
        """Find and return the tokens in the string.

        :returns: The list of tokens in the string
        """
        return LocalizedString._TOKEN_PATTERN.findall(self.value)

    def ns_localized_format(self) -> str:
        """Return the NSLocalizedString version of our call.

        :returns: The NSLocalizedString version of our call

        :raises Exception: If the language is not English
        """
        if self.language != "en":
            raise Exception(f"This should only be called for English strings: {self}")
        return (
            f"NSLocalizedStringWithDefaultValue("
            + f'@"{self.key}", @"{self.table}", @"{self.bundle}", @"{self.value}", @"{self.comment}");'
        )

    def __eq__(self, other: object) -> bool:
        """Determine if the supplied object is equal to self

        :param other: The object to compare to self

        :returns: True if they are equal, False otherwise.
        """
        if not isinstance(other, LocalizedString):
            return False

        return (
            self.value == other.value
            and self.language == other.language
            and self.comment == other.comment
            and self.key_extension == other.key_extension
            and self.table == other.table
            and self.bundle == other.bundle
            and self.key == other.key
        )

    def __ne__(self, other: object) -> bool:
        """Determine if the supplied object is unequal to self

        :param other: The object to compare to self

        :returns: True if they are unequal, False otherwise.
        """
        return not self.__eq__(other)

    def __hash__(self) -> int:
        """Calculate the hash of the object

        :returns: The hash value of the object
        """
        return hash(
            (
                self.value,
                self.language,
                self.comment,
                self.key_extension,
                self.table,
                self.bundle,
                self.key,
            )
        )

    def __repr__(self) -> str:
        """Returns a raw representation of the object which can be used to reconstruct it later.

        :returns: A raw representation of the object
        """
        return str(
            {
                "key": self.key,
                "value": self.value,
                "language": self.language,
                "table": self.table,
                "comment": self.comment,
                "bundle": self.bundle,
                "key_extension": self.key_extension,
            }
        )

    def __str__(self) -> str:
        """Generate and return the string representation of the object.

        :return: A string representation of the object
        """
        return self.__repr__()

    @staticmethod
    def from_dotstring_entries(
        *, entries: List[DotStringsEntry], language: str, table: str
    ) -> List["LocalizedString"]:
        """Convert a list of DotStringsEntry's into a list of LocalizedString's

        :param List[DotStringsEntry] entries: The DotStringsEntry's to convert
        :param str language: The language the DotStringsEntry's are in
        :param str table: The table the DotStringsEntry's are from

        :returns: A list of LocalizedStrings
        """

        output = []

        for entry in entries:
            localized = LocalizedString(
                key=entry.key,
                value=entry.value,
                language=language,
                table=table,
                comment="\n".join(entry.comments),
                key_extension=None,
                bundle="",
            )
            output.append(localized)

        return output



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dotstrings']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'dotstrings',
    'version': '1.0.2',
    'description': 'Tools for dealing with the .strings files for iOS and macOS',
    'long_description': '# dotstrings\n\nThis is a Python toolkit for interacting with the localization files for iOS and macOS.\n\n### Examples\n\n**Read in a .strings file and print the entries:**\n```python\nimport dotstrings\n\nentries = dotstrings.load("/path/to/file.strings")\n\nfor entry in entries:\n    print("Key: " + entry.key)\n    print("Value: " + entry.value)\n    print("Comments: " + "\\n".join(entry.comments))\n```\n\n# Contributing\n\nThis project welcomes contributions and suggestions.  Most contributions require you to agree to a\nContributor License Agreement (CLA) declaring that you have the right to, and actually do, grant us\nthe rights to use your contribution. For details, visit https://cla.opensource.microsoft.com.\n\nWhen you submit a pull request, a CLA bot will automatically determine whether you need to provide\na CLA and decorate the PR appropriately (e.g., status check, comment). Simply follow the instructions\nprovided by the bot. You will only need to do this once across all repos using our CLA.\n\nThis project has adopted the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/).\nFor more information see the [Code of Conduct FAQ](https://opensource.microsoft.com/codeofconduct/faq/) or\ncontact [opencode@microsoft.com](mailto:opencode@microsoft.com) with any additional questions or comments.\n',
    'author': 'Dale Myers',
    'author_email': 'dalemy@microsoft.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Microsoft/dotstrings',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
